@@ -1,6 +1,7 @@
+## Copyright 2013 by Luc Saffre.
+## License: BSD, see LICENSE for more details.
 """
-:copyright: Copyright 2013 by Luc Saffre.
-:license: BSD, see LICENSE for more details.
+The top-level package of :ref:`north`.
 """
 
 import os
@@ -111,6 +112,7 @@ class Site(Site):
     """
     If you maintain a data migration module for your application, 
     specify its name here.
+    See :ref:`datamig`.
     """
     
     loading_from_dump = False
@@ -144,19 +146,35 @@ class Site(Site):
     
     Changing this setting affects your database structure 
     if your application uses babel fields,
-    and thus require a data migration.
+    and thus require a :ref:`data migration <datamig>`.
     
-    If this is not None, Site will 
-    set the Django settings :setting:`USE_L10N` 
-    and  :setting:`LANGUAGE_CODE`.
+    If this is not `None`, Site will set the Django settings 
+    `USE_L10N <http://docs.djangoproject.com/en/dev/ref/settings/#use-l10n>`_ 
+    and
+    `LANGUAGE_CODE <http://docs.djangoproject.com/en/dev/ref/settings/#language-code>`_.
     
     """
     
+    hidden_languages = None
+    """
+    A string of django codes of languages that should be hidden.
+    
+    :ref:`welfare` uses this because the demo database has 4 
+    languages, but `nl` is currently not worth to be shown in a public 
+    demo.
+    
+    """
+        
+    
+    
     BABEL_LANGS = tuple()
     
     
-    def init_before_local(self,*args):
-        super(Site,self).init_before_local(*args)
+    def init_before_local(self,settings_globals,user_apps):
+        if isinstance(user_apps,basestring):
+            user_apps = [user_apps]
+        user_apps = user_apps + ['north']
+        super(Site,self).init_before_local(settings_globals,user_apps)
         self.update_settings(SERIALIZATION_MODULES = {
             "py" : "north.dpy",
         })
@@ -200,14 +218,17 @@ class Site(Site):
           
     def using(self,ui=None):
         """
-        
+        Adds Nort and Babel to the list of displayed versions.
         """
-        import babel
         
-        yield ("Babel",babel.__version__,"http://babel.edgewall.org/")
-        #~ yield (SETUP_INFO['name'],SETUP_INFO['version'],SETUP_INFO['url'])
+        import north
+        yield ("North",north.SETUP_INFO['version'],north.SETUP_INFO['url'])
+        
         for u in super(Site,self).using(ui): yield u
         
+        import babel
+        yield ("Babel",babel.__version__,"http://babel.edgewall.org/")
+        
    
     def apply_languages(self):
         """
@@ -216,13 +237,14 @@ class Site(Site):
         the attribute `languages` and converts it to a tuple of 
         `LanguageInfo` objects.
         
+        >>> from django.utils import translation
         >>> from north import TestSite as Site
         >>> from pprint import pprint
         >>> pprint(Site().django_settings) #doctest: +ELLIPSIS
         {'DATABASES': {'default': {'ENGINE': 'django.db.backends.sqlite3',
                                    'NAME': '...default.db'}},
          'FIXTURE_DIRS': (),
-         'INSTALLED_APPS': ('djangosite',),
+         'INSTALLED_APPS': ('north', 'djangosite'),
          'LOCALE_PATHS': (),
          'SECRET_KEY': '20227',
          'SERIALIZATION_MODULES': {'py': 'north.dpy'},
@@ -327,8 +349,14 @@ class Site(Site):
         #~ self.AVAILABLE_LANGUAGES = self.AVAILABLE_LANGUAGES + self.BABEL_LANGS
                     
         if must_set_language_code:
-            self.update_settings(LANGUAGE_CODE = self.get_default_language())
-        #~ self.update_settings(LANGUAGE_CODE = self.languages[0].django_code)
+            #~ self.update_settings(LANGUAGE_CODE = self.get_default_language())
+            self.update_settings(LANGUAGE_CODE = self.languages[0].django_code)
+            """
+            Note: LANGUAGE_CODE is what *Django* believes to be the default language.
+            This should be some variant of English ('en' or 'en-us') 
+            if you use `django.contrib.humanize`
+            https://code.djangoproject.com/ticket/20059
+            """
                     
 
     def do_site_startup(self):
@@ -339,7 +367,7 @@ class Site(Site):
         
         from django.conf import settings
         from django.utils.translation import ugettext_lazy as _
-        from north.dbutils import set_language
+        #~ from north.dbutils import set_language
         
         def langtext(code):
             for k,v in settings.LANGUAGES:
@@ -382,9 +410,10 @@ class Site(Site):
                 _add_language(lang.django_code,text)
                 
             """
-            Activate the site's default language
+            Cannot activate the site's default language
+            because some test cases in django.contrib.humanize rely on en-us as default language
             """
-            set_language(self.get_default_language())
+            #~ set_language(self.get_default_language())
 
     def get_language_info(self,code):
         """
@@ -462,7 +491,9 @@ class Site(Site):
         
     def get_default_language(self):
         """
-        The default language to use in every `north.dbutils.LanguageField`.
+        The django code of the default language to use in every 
+        :class:`LanguageField <north.dbutils.LanguageField>`.
+        
         """
         return self.DEFAULT_LANGUAGE.django_code
         
@@ -541,7 +572,6 @@ class Site(Site):
         Examples:
         
         >>> from north import TestSite as Site
-        >>> from north.dbutils import set_language
         >>> from atelier.utils import AttrDict
         >>> def testit(site_languages):
         ...     site = Site(languages=site_languages)
@@ -589,43 +619,76 @@ class Site(Site):
         #~ return l
       
 
-    def babelitem(self,**v):
+    def babelitem(self,*args,**values):
         """
         Given a dictionary with babel values, return the 
         value corresponding to the current language.
+        
         This is available in templates as a function `tr`.
         
         >>> kw = dict(de="Hallo",en="Hello",fr="Salut")
         
         >>> from north import TestSite as Site
-        >>> from north.dbutils import set_language
+        >>> from django.utils import translation
+        
+        A Site with default langueg "de":
+        
         >>> site = Site(languages="de en")
         >>> tr = site.babelitem
-        >>> set_language('de')
-        >>> tr(**kw)
+        >>> with translation.override('de'):
+        ...    tr(**kw)
         'Hallo'
         
-        >>> set_language('en')
-        >>> tr(**kw)
+        >>> with translation.override('en'):
+        ...    tr(**kw)
+        'Hello'
+
+        If the current language is not found in the specified `values`,
+        then it returns the site's default language:
+        
+        >>> with translation.override('jp'):
+        ...    tr(en="Hello",de="Hallo",fr="Salut")
+        'Hallo'
+        
+        Another way is to specify an explicit default value using a
+        positional argument. In that case the language's default language
+        doesn'n matter:
+        
+        >>> with translation.override('jp'):
+        ...    tr("Hello",de="Hallo",fr="Salut")
         'Hello'
         
-        >>> set_language('jp')
-        >>> tr(**kw)
+        >>> with translation.override('de'):
+        ...     tr("Hello",de="Hallo",fr="Salut")
         'Hallo'
         
+        You may not specify more than one default value:
+        
+        >>> tr("Hello","Hallo")
+        Traceback (most recent call last):
+        ...
+        ValueError: ('Hello', 'Hallo') is more than 1 default value.
+        
+        
         >>> 
         """
         from django.utils import translation
-        info = self.language_dict.get(translation.get_language(),self.DEFAULT_LANGUAGE)
-        #~ lng = translation.get_language()
-        #~ print lng,v
-        #~ lng = LANG or DEFAULT_LANGUAGE
-        if info == self.DEFAULT_LANGUAGE:
-            return v.get(info.name)
-        x = v.get(info.name,None)
-        if x is None:
-            return v.get(self.DEFAULT_LANGUAGE.name)
-        return x
+        if len(args) == 0:
+            info = self.language_dict.get(translation.get_language(),self.DEFAULT_LANGUAGE)
+            default_value = None
+            if info == self.DEFAULT_LANGUAGE:
+                return values.get(info.name)
+            x = values.get(info.name,None)
+            if x is None:
+                return values.get(self.DEFAULT_LANGUAGE.name)
+            return x
+        elif len(args) == 1:
+            info = self.language_dict.get(translation.get_language(),None)
+            if info is None:
+                return args[0]
+            default_value = args[0]
+            return values.get(info.name,default_value)
+        raise ValueError("%(values)s is more than 1 default value." % dict(values=args))
         
     # babel_get(v) = babelitem(**v)
             
@@ -650,8 +713,8 @@ class Site(Site):
         
         Examples:
         
+        >>> from django.utils import translation
         >>> from north import TestSite as Site
-        >>> from north.dbutils import set_language
         >>> from atelier.utils import AttrDict
         >>> def testit(site_languages):
         ...     site = Site(languages=site_languages)
@@ -660,12 +723,12 @@ class Site(Site):
         
         
         >>> site,obj = testit('de en')
-        >>> set_language('de')
-        >>> site.babelattr(obj,'name')
+        >>> with translation.override('de'):
+        ...     site.babelattr(obj,'name')
         'Hallo'
         
-        >>> set_language('en')
-        >>> site.babelattr(obj,'name')
+        >>> with translation.override('en'):
+        ...     site.babelattr(obj,'name')
         'Hello'
         
         If the object has no translation for a given language, 
@@ -674,14 +737,14 @@ class Site(Site):
         The language exists on the site, but the object has no translation for it:
         
         >>> site,obj = testit('en es')
-        >>> set_language('es')
-        >>> site.babelattr(obj,'name')
+        >>> with translation.override('es'):
+        ...     site.babelattr(obj,'name')
         'Hello'
         
         Or a language has been activated which doesn't exist on the site:
        
-        >>> set_language('fr')
-        >>> site.babelattr(obj,'name')
+        >>> with translation.override('fr'):
+        ...     site.babelattr(obj,'name')
         'Hello'
         
         """
@@ -716,12 +779,11 @@ class TestSite(Site):
     >> from north import TestSite as Site
     >> Site(...)
     
-    
-    
     """
     def __init__(self,*args,**kwargs):
         kwargs.update(no_local=True)
         g = dict(__file__=__file__)
+        g.update(SECRET_KEY="20227") # see :djangoticket:`20227`
         super(TestSite,self).__init__(g,*args,**kwargs)
         
     #~ def run_djangosite_local(self):
