@@ -1,10 +1,10 @@
 # -*- coding: UTF-8 -*-
+## Copyright 2009-2013 by Luc Saffre.
+## License: BSD, see file LICENSE for more details.
+
 """
 Documented in :ref:`dpy`.
 
-:copyright: Copyright 2009-2013 by Luc Saffre.
-:license: BSD, see LICENSE for more details.
-
 """
 
 import logging
@@ -28,6 +28,7 @@ from django.contrib.contenttypes.models import ContentType
 from django.contrib.sessions.models import Session
 from django.utils.encoding import smart_unicode, is_protected_type, force_unicode
 from django.utils.importlib import import_module
+from django.utils import translation
 
 from djangosite.dbutils import obj2str, sorted_models_list, full_model_name
 from north import dbutils
@@ -117,13 +118,13 @@ class Serializer(base.Serializer):
         self.selected_fields = options.get("fields")
         self.use_natural_keys = options.get("use_natural_keys", False)
         if self.write_preamble:
-            self.stream.write('# -*- coding: UTF-8 -*-\n')
             #~ name,current_version,url = settings.SITE.using().next()
             current_version = settings.SITE.version
             if '+' in current_version:
                 logger.warning(
                     "Dumpdata from intermediate version %s" % current_version)
             
+            self.stream.write('# -*- coding: UTF-8 -*-\n')
             self.stream.write('''\
 """
 This is a `Python dump <http://north.lino-framework.org>`_
@@ -440,7 +441,8 @@ class FakeDeserializedObject(base.DeserializedObject):
             m = getattr(obj,'before_dumpy_save',None)
             if m is not None:
                 m()
-            obj.full_clean()
+            if not self.deserializer.quick:
+                obj.full_clean()
             obj.save(*args,**kw)
             logger.debug("%s has been saved" % obj2str(obj))
             self.deserializer.register_success()
@@ -496,13 +498,15 @@ class FlushDeferredObjects:
     """
     pass
 
-class DpyDeserializer:
+class LoaderBase:
+    
+    quick = False
     
     def __init__(self):
-        #~ logger.info("20120225 DpyDeserializer.__init__()")
+        #~ logger.info("20120225 DpyLoader.__init__()")
         self.save_later = {}
         self.saved = 0
-        #~ self.count = 0
+        self.count_objects = 0
 
   
     def flush_deferred_objects(self):
@@ -526,12 +530,50 @@ class DpyDeserializer:
                     #~ self.save_later.append(obj)
             logger.info("Saved %d instances.",self.saved)
             
+    def expand(self,obj):
+        if obj is None:
+            pass # ignore None values
+        elif obj is FlushDeferredObjects:
+            self.flush_deferred_objects()
+        elif isinstance(obj,models.Model):
+            yield FakeDeserializedObject(self,obj)
+        elif hasattr(obj,'__iter__'):
+        #~ if type(obj) is GeneratorType:
+            #~ logger.info("20120225 expand iterable %r",obj)
+            for o in obj: 
+                for so in self.expand(o): 
+                    yield so
+        #~ elif isinstance(obj,MtiChildWrapper):
+            # the return value of create_mti_child()
+            #~ yield FakeDeserializedObject(self,obj)
+            #~ obj.deserializer = self
+            #~ yield obj
+        else:
+            logger.warning("Ignored unknown object %r",obj)
+    
+        
+    def register_success(self):
+        self.saved += 1
+        self.count_objects += 1
+        
+    def register_failure(self,obj,e):
+        msg = force_unicode(e)
+        d = self.save_later.setdefault(obj.object.__class__,{})
+        l = d.setdefault(msg,[])
+        if len(l) == 0:
+            logger.info("Deferred %s : %s",obj2str(obj.object),msg)
+        l.append(obj)
+        
+class DpyDeserializer(LoaderBase):
+    
     def deserialize(self,fp, **options):
-        #~ logger.info("20120225 DpyDeserializer.deserialize()")
+        #~ logger.info("20120225 DpyLoader.deserialize()")
         if isinstance(fp, basestring):
             raise NotImplementedError
         #~ dbutils.set_language(settings.SITE.DEFAULT_LANGUAGE.django_code)
-        dbutils.set_language()
+        #~ dbutils.set_language()
+        
+        translation.activate(settings.SITE.get_default_language())
         
         #~ self.count += 1
         fqname = 'north.dpy_tmp_%s' % hash(self)
@@ -556,35 +598,16 @@ class DpyDeserializer:
                 
     def deserialize_module(self,module, **options):
         
-        def expand(obj):
-            if obj is None:
-                pass # ignore None values
-            elif obj is FlushDeferredObjects:
-                self.flush_deferred_objects()
-            elif isinstance(obj,models.Model):
-                yield FakeDeserializedObject(self,obj)
-            elif hasattr(obj,'__iter__'):
-            #~ if type(obj) is GeneratorType:
-                #~ logger.info("20120225 expand iterable %r",obj)
-                for o in obj: 
-                    for so in expand(o): 
-                        yield so
-            #~ elif isinstance(obj,MtiChildWrapper):
-                # the return value of create_mti_child()
-                #~ yield FakeDeserializedObject(self,obj)
-                #~ obj.deserializer = self
-                #~ yield obj
-            else:
-                logger.warning("Ignored unknown object %r",obj)
-                
         if not hasattr(module,'objects'):
             #~ raise Exception("%s has no attribute 'objects'" % fp.name)
             raise Exception("Fixture %s has no attribute 'objects'" % module.__name__)
+            
         empty_fixture = True
         for obj in module.objects():
-            for o in expand(obj): 
+            for o in self.expand(obj): 
                 empty_fixture = False
                 yield o
+            
         if empty_fixture:
             if SUPPORT_EMPTY_FIXTURES:
                 yield DummyDeserializedObject() # avoid Django interpreting empty fixtures as an error
@@ -637,17 +660,16 @@ See <https://code.djangoproject.com/ticket/18213>.
         if hasattr(module,'after_load'):
             module.after_load()
         
+       
+    
+class DpyLoader(LoaderBase):        
+        
+    def save(self,obj):
+        for o in self.expand(obj):
+            o.try_save()
+        
 
-    def register_success(self):
-        self.saved += 1
         
-    def register_failure(self,obj,e):
-        msg = force_unicode(e)
-        d = self.save_later.setdefault(obj.object.__class__,{})
-        l = d.setdefault(msg,[])
-        if len(l) == 0:
-            logger.info("Deferred %s : %s",obj2str(obj.object),msg)
-        l.append(obj)
 
 def Deserializer(fp, **options):
     """
