@@ -1,25 +1,23 @@
-"""
-:copyright: Copyright 2013 by Luc Saffre.
-:license: BSD, see LICENSE for more details.
-"""
+## Copyright 2013 by Luc Saffre.
+## License: BSD, see LICENSE for more details.
 
 SETUP_INFO = dict(
   name = 'North',
-  version = '0.1.6', # released 20130813
+  version = '0.1.7', 
   install_requires = ['djangosite','Babel'],
   test_suite = 'tests',
   description = "Another way to migrate Django databases",
   #~ long_description=open('README.txt').read(),
   long_description="""\
-North is a Python code serializer/deserializer for Django which lets 
-you write intelligent fixtures and generate database dumps. 
+North is a Django extension which lets you write database fixtures and 
+make database dumps in Python.
 You can use it to provide automated database migrations, which makes 
 it an alternative to `South <http://south.aeracode.org/>`__
 
 North includes an optional single-table solution for handling multilingual 
 database content.
 
-North doesn't require any database model, it is not even a Django app. 
+North doesn't require any database model. 
 Basic usage in your `settings.py` file is::
 
   from north import Site
@@ -41,7 +39,7 @@ for certain Django settings, including `INSTALLED_APPS` and
 `SERIALIZATION_MODULES`.
 """,
   license = 'Free BSD',
-  packages = ['north','north.demo'],
+  packages = ['north','north.dbutils', 'north.demo','north.management','north.management.commands'],
   author = 'Luc Saffre',
   author_email = 'luc.saffre@gmail.com',
   url = "http://north.lino-framework.org",
